import { getCurrentHub } from '@sentry/core';
import { app, powerMonitor, screen, } from 'electron';
/** Electron integration that cleans up the event. */
export class Electron {
    constructor() {
        /**
         * @inheritDoc
         */
        this.name = Electron.id;
    }
    /**
     * @inheritDoc
     */
    setupOnce() {
        this._instrumentBreadcrumbs('app', app);
        app.once('ready', () => {
            // We can't access these until 'ready'
            this._instrumentBreadcrumbs('Screen', screen);
            this._instrumentBreadcrumbs('PowerMonitor', powerMonitor);
        });
        app.on('web-contents-created', (_, contents) => {
            // SetImmediate is required for contents.id to be correct
            // https://github.com/electron/electron/issues/12036
            setImmediate(() => {
                if (contents.isDestroyed()) {
                    return;
                }
                const options = getCurrentHub().getClient().getOptions();
                const customName = options.getRendererName && options.getRendererName(contents);
                this._instrumentBreadcrumbs(customName || `WebContents[${contents.id}]`, contents, [
                    'dom-ready',
                    'load-url',
                    'destroyed',
                ]);
            });
        });
    }
    /**
     * Hooks into the Electron EventEmitter to capture breadcrumbs for the
     * specified events.
     */
    _instrumentBreadcrumbs(category, emitter, events = []) {
        const emit = emitter.emit.bind(emitter);
        emitter.emit = (event, ...args) => {
            if (events.length === 0 || events.indexOf(event) > -1) {
                const breadcrumb = {
                    category: 'electron',
                    message: `${category}.${event}`,
                    timestamp: new Date().getTime() / 1000,
                    type: 'ui',
                };
                const self = getCurrentHub().getIntegration(Electron);
                if (self) {
                    getCurrentHub().addBreadcrumb(breadcrumb);
                }
            }
            return emit(event, ...args);
        };
    }
}
/**
 * @inheritDoc
 */
Electron.id = 'Electron';
//# sourceMappingURL=electron.js.map